package me.unurled.sacredrealms.sr.components.cutscene;

import static me.unurled.sacredrealms.sr.utils.Component.textComp;
import static me.unurled.sacredrealms.sr.utils.CutsceneUtil.showBlackScreen;
import static me.unurled.sacredrealms.sr.utils.Logger.error;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import me.unurled.sacredrealms.sr.SR;
import me.unurled.srcore.api.Manager;
import net.kyori.adventure.text.Component;
import org.bukkit.Bukkit;
import org.bukkit.GameMode;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.scheduler.BukkitTask;
import org.jetbrains.annotations.NotNull;

/**
 * Cutscene class
 *
 * <p>Represents a cutscene
 *
 * <pre>{@code
 * {
 *   "id": "cutscene1",
 *   "name": "Cutscene 1",
 *   "markers": [1, 2, 3], // Marker IDs
 *   "start": {
 *      "world": "world",
 *      "x": 0,
 *      "y": 0,
 *      "z": 0,
 *      "yaw": 0,
 *      "pitch": 0
 *   },
 *   "end": {
 *      "world": "world",
 *      "x": 0,
 *      "y": 0,
 *      "z": 0,
 *      "yaw": 0,
 *      "pitch": 0
 *    }
 *    "blackBorders": true,
 *    "startFadeToBlack": true,
 *    "endFadeToBlack": true
 * }
 * }</pre>
 */
public class Cutscene {

  private final String id;
  private final String name;
  private final List<Marker> markers;
  private final List<Integer> markerIds;
  private CutsceneManager cutsceneManager;
  private Location start;
  private Location end;
  private Map<UUID, BukkitTask> tasks;
  private boolean blackBorders;
  private boolean startFadeToBlack;
  private boolean endFadeToBlack;

  public Cutscene(String id, String name) {
    this.id = id;
    this.name = name;
    this.markers = new ArrayList<>();
    this.markerIds = new ArrayList<>();
    this.tasks = new HashMap<>();
    cutsceneManager = Manager.getInstance(CutsceneManager.class);
    if (cutsceneManager == null) {
      error("CutsceneManager is null, could not create a Cutscene");
    }
  }

  public Cutscene(@NotNull String id, String name, @NotNull ArrayList<Marker> markers) {
    this.id = id.toUpperCase();
    this.name = name;
    this.markers = markers;
    markerIds = new ArrayList<>();
    for (Marker marker : markers) {
      markerIds.add(marker.getId());
    }
    if (markers.isEmpty()) return;
    if (markers.getFirst().getFrames().isEmpty()) return;
    start = markers.getFirst().getFrames().getFirst().getLocation();
    end = markers.getLast().getFrames().getLast().getLocation();
    this.tasks = new HashMap<>();
    cutsceneManager = Manager.getInstance(CutsceneManager.class);
    if (cutsceneManager == null) {
      error("CutsceneManager is null, could not create a Cutscene");
    }
  }

  public void setMarkers() {
    for (Integer i : markerIds) {
      markers.add(Marker.getMarker(i));
    }
  }

  public void addMarker(Marker marker) {
    this.markers.add(marker);
  }

  public String getId() {
    return this.id.toUpperCase();
  }

  public String getName() {
    return this.name;
  }

  public List<Marker> getMarkers() {
    return this.markers;
  }

  public Location getStart() {
    return this.start;
  }

  public void setStart(Location start) {
    this.start = start;
  }

  public Location getEnd() {
    return this.end;
  }

  public void setEnd(Location end) {
    this.end = end;
  }

  public void addMarkerId(Integer id) {
    this.markerIds.add(id);
  }

  public long time() {
    long time = 0;
    for (Marker marker : this.markers) {
      if (marker.getOverAllTime() != null) time += marker.getOverAllTime();
    }
    return time;
  }

  public void play(@NotNull Player p) {
    if (cutsceneManager.isPlayingCutscene(p)) {
      p.sendMessage(textComp("You are already playing a cutscene!"));
    }
    cutsceneManager.addCurrentPlayingCutscene(this, p);

    // hide toolbar and make it spectate entity
    p.setGameMode(GameMode.SPECTATOR);

    // set black borders
    if (blackBorders) {
      p.getInventory().setHelmet(new ItemStack(Material.PUMPKIN));
      p.updateInventory();
    }

    if (startFadeToBlack) {
      showBlackScreen(p, 2, 1, 1);
    }

    BukkitTask task =
        Bukkit.getScheduler()
            .runTaskAsynchronously(
                SR.getPlugin(),
                () -> {
                  int finalI = 0;
                  for (Marker marker : markers) {
                    for (int i = 0; i < marker.getFrames().size(); i++) {
                      int finalI1 = i;
                      Bukkit.getScheduler()
                          .runTaskLater(
                              SR.getPlugin(),
                              () -> {
                                p.teleport(marker.getFrames().get(finalI1).getLocation());
                                marker.getFrames().get(finalI1).play();
                              },
                              (marker.getTimeBetweenFrames() * finalI) / 50);
                      finalI++;
                    }
                  }
                  if (endFadeToBlack) {
                    Bukkit.getScheduler()
                        .runTask(SR.getPlugin(), () -> showBlackScreen(p, 2, 1, 1));
                  }
                });
    Bukkit.getScheduler()
        .runTaskAsynchronously(
            SR.getPlugin(),
            () -> {
              try {
                Thread.sleep(time());
              } catch (InterruptedException e) {
                error("Error while playing cutscene: " + e.getMessage());
              }
              Bukkit.getScheduler().runTask(SR.getPlugin(), () -> stop(p));
            });
    tasks.put(p.getUniqueId(), task);
  }

  public void stop(UUID player) {
    Player p = Bukkit.getPlayer(player);
    if (p == null) return;
    stop(p);
  }

  public void stop(@NotNull Player p) {
    if (cutsceneManager.isPlayingCutscene(p)) {
      if (cutsceneManager.getPlayingCutscene(p).equals(this.id)) {
        cutsceneManager.removeCurrentPlayingCutscene(this, p);
        tasks.get(p.getUniqueId()).cancel();
        p.sendMessage(Component.text("You stopped playing the cutscene!"));
      }
    } else {
      p.sendMessage(Component.text("You are not playing a cutscene!"));
    }
  }

  public void calculateMarkers() {
    if (markers.isEmpty()) return;
    for (int i = 0; i < markers.size(); i++) {
      if (i < markers.size() - 1) {
        markers.get(i).calculateFrames(markers.get(i + 1));
      } else {
        ArrayList<Frame> frames = new ArrayList<>();
        frames.add(new Frame(markers.get(i).getStart()));
        markers.get(i).setFrames(frames);
      }
    }
  }
}
