package me.unurled.sacredrealms.sr.components.pack;

import static me.unurled.sacredrealms.sr.utils.File.getFileChecksum;
import static me.unurled.sacredrealms.sr.utils.File.zipDirectory;
import static me.unurled.sacredrealms.sr.utils.File.zipFile;
import static me.unurled.sacredrealms.sr.utils.Logger.error;
import static me.unurled.sacredrealms.sr.utils.Logger.log;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.util.UUID;
import java.util.zip.ZipOutputStream;
import me.unurled.sacredrealms.sr.SR;
import me.unurled.sacredrealms.sr.components.pack.hosts.HostingProvider;
import me.unurled.sacredrealms.sr.components.pack.hosts.RustProvider;
import me.unurled.sacredrealms.sr.config.Settings;
import me.unurled.sacredrealms.sr.data.DataHandler;
import me.unurled.sacredrealms.sr.data.DataManager;
import me.unurled.srcore.api.Manager;
import org.bukkit.Bukkit;
import org.jetbrains.annotations.NotNull;

public class ResourcePackManager extends Manager {

  private static final String RESOURCE_PACK_ZIP = "resource_pack.zip";
  private final HostingProvider hostingProvider;

  public ResourcePackManager() {
    this.hostingProvider =
        new RustProvider(
            Settings.RESOURCE_PACK_URL.toString(), Settings.RESOURCE_PACK_API.toString());
  }

  @NotNull
  private static File packFile(String fileName) {
    try {
      new File(SR.getPlugin().getDataFolder(), "pack").mkdirs();

      String[] filesInclude = {"assets", "pack.mcmeta"};

      FileOutputStream fos =
          new FileOutputStream(new File(SR.getPlugin().getDataFolder(), fileName));
      ZipOutputStream zipOut = new ZipOutputStream(fos);

      zipOut.setLevel(9);

      for (String filePath : filesInclude) {
        File fileToZip = new File(SR.getPlugin().getDataFolder() + "/pack", filePath);
        if (fileToZip.isDirectory()) {
          zipDirectory(fileToZip, fileToZip.getName(), zipOut);
        } else {
          zipFile(fileToZip, fileToZip.getName(), zipOut);
        }
      }
      zipOut.close();
      fos.close();
      log("ZIP file created successfully!");
    } catch (IOException e) {
      error("Failed to create resource pack");
    }
    return new File(SR.getPlugin().getDataFolder(), fileName);
  }

  /** Save the data */
  @Override
  public void saveData() {
    super.saveData();
    DataManager dm = Manager.getInstance(DataManager.class);
    DataHandler dh = dm.getDataHandler();
    dh.set("pack.url", hostingProvider.getMinecraftPackURL());
    dh.set("pack.uuid", hostingProvider.getPackUUID().toString());
    dh.set("pack.sha1", hostingProvider.getPackUUID().toString());
  }

  /** Load the data */
  @Override
  public void loadData() {
    super.loadData();
    DataManager dm = Manager.getInstance(DataManager.class);
    DataHandler dh = dm.getDataHandler();
    String packUUID = dh.get("pack.uuid");
    if (packUUID == null) {
      return;
    }
    hostingProvider.setPackUUID(UUID.fromString(packUUID));
    hostingProvider.setPackURL(dh.get("pack.url"));
    hostingProvider.setSHA1(dh.get("pack.sha1").getBytes());
  }

  /**
   * upload a file to the hosting provider and return the url
   *
   * @param file the file to upload
   */
  public void uploadAsync(final File file) {
    Bukkit.getScheduler()
        .runTaskAsynchronously(
            SR.getPlugin(),
            () -> {
              // start the call
              boolean success;
              try {
                success = hostingProvider.uploadPack(file);
              } catch (Exception e) {
                error("Failed to upload resource pack (exception) " + e.getMessage());
                return;
              }
              if (!success) {
                error("Failed to upload resource pack.");
              }
            });
  }

  /** Load the manager */
  @Override
  public void load() {
    super.load();
    String zipPath = SR.getPlugin().getDataFolder() + "/" + RESOURCE_PACK_ZIP;

    Bukkit.getScheduler()
        .runTaskAsynchronously(
            SR.getPlugin(),
            () -> {
              if (new File(SR.getPlugin().getDataFolder(), "pack").exists()
                  && new File(zipPath).exists()) {
                log("Resource pack exists");
                // create a new zip with files

                File tmpFile = packFile("tmp.zip");
                try {
                  String tmpSHA = getFileChecksum(tmpFile.getAbsolutePath());
                  String rpSHA = getFileChecksum(zipPath);

                  if (tmpSHA.equals(rpSHA)) {
                    log("Resource pack is up to date");
                    return;
                  } else {
                    log("Resource pack is outdated");
                    log(tmpSHA + " " + rpSHA);
                  }
                } catch (IOException | NoSuchAlgorithmException e) {
                  error("Failed to check resource pack hash");
                }
              }
              // create the resource pack
              File rp = packFile(RESOURCE_PACK_ZIP);

              // upload the resource pack
              uploadAsync(rp);
            });
  }

  public HostingProvider getHostingProvider() {
    return hostingProvider;
  }
}
