package me.unurled.sacredrealms.sr.components.pack.hosts;

import static me.unurled.sacredrealms.sr.utils.Logger.error;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.UUID;
import me.unurled.sacredrealms.sr.utils.HTTPRequestMultipartBody;
import org.jetbrains.annotations.NotNull;

public class RustProvider implements HostingProvider {

  public static final String FAILED = "Failed to upload pack to Rust server: ";
  public static final String HTTPS = "https://";
  private final String address;
  private final String APIKEY;
  private String packUrl;
  private String minecraftPackURL;
  private String sha1;
  private UUID packUUID;

  public RustProvider(@NotNull String address, @NotNull String apiKey) {
    String address1;
    if (address.startsWith(HTTPS) || address.startsWith("http://")) {
      address1 = address;
    } else {
      address1 = HTTPS + address;
    }
    address1 += address.endsWith("/") ? "" : "/";
    this.address = address1;
    this.APIKEY = apiKey;
  }

  @Override
  public boolean uploadPack(File resourcePack) throws InterruptedException {
    try (HttpClient httpClient = HttpClient.newHttpClient()) {
      HTTPRequestMultipartBody multipartBody =
          new HTTPRequestMultipartBody.Builder()
              .addPart("file", resourcePack, null, resourcePack.getName())
              .build();

      HttpRequest httpRequest =
          HttpRequest.newBuilder()
              .header(
                  "Content-Type", "multipart/form-data; boundary=" + multipartBody.getBoundary())
              .header("API_KEY", APIKEY)
              .uri(new URI(this.address + "post"))
              .POST(HttpRequest.BodyPublishers.ofByteArray(multipartBody.getBody()))
              .build();

      HttpResponse<String> response =
          httpClient.send(httpRequest, HttpResponse.BodyHandlers.ofString());

      if (response.statusCode() != 200) {
        error(FAILED + response.body());
        return false;
      }

      String stringRes = response.body();

      if (stringRes.startsWith("[") && stringRes.endsWith("]")) {
        stringRes = stringRes.substring(1, stringRes.length() - 1);
      }

      String[] split = stringRes.split(",");
      if (split.length != 2) {
        error(FAILED + response.body());
        return false;
      }

      String path = split[0].split(":")[1].replace("\"", "").trim();
      sha1 = split[1].split(":")[1].replace("\"", "").trim().replace("}", "");
      packUrl = this.address + "files/" + path;

      minecraftPackURL = packUrl.replace(HTTPS, "http://");

      packUUID = UUID.nameUUIDFromBytes(sha1.getBytes());

    } catch (IOException | URISyntaxException e) {
      error(FAILED + e.getMessage());
      return false;
    } catch (InterruptedException e) {
      error(FAILED + e.getMessage());
      throw e;
    }
    return true;
  }

  @Override
  public String getPackURL() {
    return packUrl;
  }

  @Override
  public void setPackURL(String packURL) {
    this.packUrl = packURL;
  }

  @Override
  public String getMinecraftPackURL() {
    return minecraftPackURL;
  }

  @Override
  public void setMinecraftPackURL(String minecraftPackURL) {
    this.minecraftPackURL = minecraftPackURL;
  }

  @Override
  public byte[] getSHA1() {
    int len = sha1.length();
    byte[] data = new byte[len / 2];
    for (int i = 0; i < len; i += 2) {
      data[i / 2] =
          (byte)
              ((Character.digit(sha1.charAt(i), 16) << 4)
                  + Character.digit(sha1.charAt(i + 1), 16));
    }
    return data;
  }

  @Override
  public void setSHA1(byte[] sha1) {
    this.sha1 = new String(sha1);
  }

  @Override
  public String getOriginalSHA1() {
    return sha1;
  }

  @Override
  public void setOriginalSHA1(String originalSHA1) {
    this.sha1 = originalSHA1;
  }

  @Override
  public UUID getPackUUID() {
    return packUUID;
  }

  @Override
  public void setPackUUID(UUID packUUID) {
    this.packUUID = packUUID;
  }
}
